import aiosched


async def sender(c: aiosched.MemoryChannel, n: int):
    for i in range(n):
        await c.write(str(i))
        print(f"Sent {i}")
    await c.close()
    print("Sender done")


async def receiver(c: aiosched.MemoryChannel):
    while True:
        if not await c.pending() and c.closed:
            print("Receiver done")
            break
        item = await c.read()
        print(f"Received {item}")
        await aiosched.sleep(1)


async def main(channel: aiosched.MemoryChannel, n: int):
    print("Starting sender and receiver")
    async with aiosched.create_pool() as ctx:
        await ctx.spawn(sender, channel, n)
        await ctx.spawn(receiver, channel)
    print("All done!")


aiosched.run(
    main, aiosched.MemoryChannel(2), 5, debugger=()
)  # 2 is the max size of the channel
